/*
  naturals.cc

  Copyright (C) 2015 Taiji Yamada <taiji@aihara.co.jp>
*/
#include <iostream>
#include <string>
#include <libgen.h>
#include <cmath>
#include <boost/lexical_cast.hpp>
#include <boost/math/common_factor.hpp>
#include "prime_util.hh"

enum naturals_type {
  natural,
  odd,
  even,
  singly_even,
  doubly_even,
  composite,
  prime,
  sophie_germain_prime,
  interprime,
  twin_prime,
  cousin_primes,
  sexy_primes,
  square_free,
  cube_free,
  perfect_square,
  perfect_power,
  powerful,
  //
  highlycomposite,
  highlyabundant,
  superabundant,
  abundant,
  primitive_abundant,
  primitive_abundant_have_no_abundant_proper_divisor,
  non_prime_unusual,
  //
  pseudoperfect, semiperfect,
  primitive_pseudoperfect, primitive_semiperfect,
  perfect,
  multiperfect,
  deficient,
  superperfect,
  hemiperfect,
  refactorable,
  unitary_perfect,
  primary_pseudoperfect,
  weird,
  practical,
  ruth_aaron_pair,
  primorial_ruth_aaron_pair,
  unusual,
  //
  arithmetic,
  harmonic_divisor,
  round_number,
  lucas_carmichael,
  semiprime,
  sphenic,
  amicable,
  betrothed,
  augumented_amicable,
  friendly,
  //
  mersenne,
  mersenne_prime,
  wilson_quotient,
  wilson_prime,
  //
  fortunate,
  hilbert,
  pronic,
  power_of_two,
  cullen,
  cullen_prime,
  woodall,
  woodall_prime,
  regular,
  factorial_prime,
  euclid,
  euclid_prime,
  kummer,
  kummer_prime,
  primorial_prime,
  triangular,
  triangular_pyramidal, tetrahedral,
  pentatope,
  pentagonal,
  square_triangular,
  squared_triangular,
  lazy_caterer,
  jacobsthal,
  jacobsthal_lucas,
  kynea,
  prime_kynea,
  thabit,
  prime_thabit,
  fermat,
  catalan,
  derangement,
  nontotient,
  noncototient,
  perfect_totient,
  highly_totient,
  highly_cototient,
  odd_semifactrial,
  even_semifactrial,
  //
  fibonacci,
  leonardo,
  lucas,
  pell,
  perrin,
  padovan,
  motzkin,
  schreder_hipparchus,
  sylvester,
  telephone,
  alternating_factorial,
  hyperfactorial,
  exponential_factorial,
};
#define S(x) #x
const char *naturals_labels[] = {
  S(natural),
  S(odd),
  S(even),
  S(singly_even),
  S(doubly_even),
  S(composite),
  S(prime),
  S(sophie_germain_prime),
  S(interprime),
  S(twin_prime),
  S(cousin_primes),
  S(sexy_primes),
  S(square_free),
  S(cube_free),
  S(perfect_square),
  S(perfect_power),
  S(powerful),
  //
  S(highlycomposite),
  S(highlyabundant),
  S(superabundant),
  S(abundant),
  S(primitive_abundant),
  S(primitive_abundant_have_no_abundant_proper_divisor),
  S(non_prime_unusual),
  //
  S(pseudoperfect), S(semiperfect),
  S(primitive_pseudoperfect), S(primitive_semiperfect),
  S(perfect),
  S(multiperfect),
  S(deficient),
  S(superperfect),
  S(hemiperfect),
  S(refactorable),
  S(unitary_perfect),
  S(primary_pseudoperfect),
  S(weird),
  S(practical),
  S(ruth_aaron_pair),
  S(primorial_ruth_aaron_pair),
  S(unusual),
  //
  S(arithmetic),
  S(harmonic_divisor),
  S(round_number),
  S(lucas_carmichael),
  S(semiprime),
  S(sphenic),
  S(amicable),
  S(betrothed),
  S(augumented_amicable),
  S(friendly),
  //
  S(mersenne),
  S(mersenne_prime),
  S(wilson_quotient),
  S(wilson_prime),
  //
  S(fortunate),
  S(hilbert),
  S(pronic),
  S(power_of_two),
  S(cullen),
  S(cullen_prime),
  S(woodall),
  S(woodall_prime),
  S(regular),
  S(factorial_prime),
  S(euclid),
  S(euclid_prime),
  S(kummer),
  S(kummer_prime),
  S(primorial_prime),
  S(triangular),
  S(triangular_pyramidal), S(tetrahedral),
  S(pentatope),
  S(pentagonal),
  S(square_triangular),
  S(squared_triangular),
  S(lazy_caterer),
  S(jacobsthal),
  S(jacobsthal_lucas),
  S(kynea),
  S(prime_kynea),
  S(thabit),
  S(prime_thabit),
  S(fermat),
  S(catalan),
  S(derangement),
  S(nontotient),
  S(noncototient),
  S(perfect_totient),
  S(highly_totient),
  S(highly_cototient),
  S(odd_semifactrial),
  S(even_semifactrial),
  //
  S(fibonacci),
  S(leonardo),
  S(lucas),
  S(pell),
  S(perrin),
  S(padovan),
  S(motzkin),
  S(schreder_hipparchus),
  S(sylvester),
  S(telephone),
  S(alternating_factorial),
  S(hyperfactorial),
  S(exponential_factorial),
};
#undef S

int usage(std::ostream &os, int argc, char *argv[])
{
  (void)argc;
  os << "Usage:\n\t" << basename(argv[0]) << " [-type name] [-l lower] [-u upper]\n";
  os << "\ntype names:" << std::endl;
  for (size_t i=0; i<sizeof(naturals_labels)/sizeof(*naturals_labels); ++i)
    os << "\t" << naturals_labels[i] << std::endl;
  return os == std::cerr ? 1 : 0;
}
int main(int argc, char *argv[])
{
  enum naturals_type type(natural);
  int a;
  isprime_bit_table_class::huge_uint_type n, l(1), u(100);

  for (a=1; a<argc; ++a)
    if (std::string(argv[a]) == "-h")
      return usage(std::cout, argc, argv);
    else if (std::string(argv[a]) == "-type" && a+1 < argc) {
      for (size_t i=0; i<sizeof(naturals_labels)/sizeof(*naturals_labels); ++i)
	if (std::string(argv[a+1]) == naturals_labels[i]) {
	  ++a;
	  type = static_cast<naturals_type>(i);
	  break;
	}
      switch (type) {
      case pronic:
      case triangular:
      case triangular_pyramidal: case tetrahedral:
      case pentatope:
      case pentagonal:
      case square_triangular:
      case squared_triangular:
      case jacobsthal:
      case derangement:
      case fibonacci:
      case pell:
      case perrin:
	l = 0;
	break;
      default:
	l = 1;
	break;
      }
    }
    else if (std::string(argv[a]) == "-l" && a+1 < argc)
      l = boost::lexical_cast<isprime_bit_table_class::huge_uint_type>(argv[++a]);
    else if (std::string(argv[a]) == "-u" && a+1 < argc)
      u = boost::lexical_cast<isprime_bit_table_class::huge_uint_type>(argv[++a]);
    else
      break;

  isprime_bit_table_class isprime_bit_table(u);

  n = l;
  switch (type) {
  case natural:
    for (; n < u; ++n)
      std::cout << n << std::endl;
    break;
  case odd:
    if (n % 2 == 0) ++n;
    for (; n < u; n+=2)
      std::cout << n << std::endl;
    break;
  case even:
    if (n % 2 == 1) ++n;
    for (; n < u; n+=2)
      std::cout << n << std::endl;
    break;
  case singly_even:
    if (n % 2 == 1) ++n;
    if (n % 4 == 0) n += 2;
    for (; n < u; n+=4)
      std::cout << n << std::endl;
    break;
  case doubly_even:
    n += 4 - n % 4;
    for (; n < u; n+=4)
      std::cout << n << std::endl;
    break;
  case composite:
    n = isprime_bit_table.firstcomposite(n);
    for (; n < u; n = isprime_bit_table.nextcomposite(n))
      std::cout << n << std::endl;
    break;
  case prime:
    n = isprime_bit_table.firstprime(n);
    for (; n < u; n = isprime_bit_table.nextprime(n))
      std::cout << n << std::endl;
    break;
  case sophie_germain_prime:
    {
      n = isprime_bit_table.firstprime(n);
      for (; n < u; n = isprime_bit_table.nextprime(n)) {
	isprime_bit_table_class::huge_uint_type m(n*2 + 1);
	if (!(m < isprime_bit_table.get_upper())) throw std::length_error("isprime_bit_table: over the length");
	if (isprime_bit_table[m]) std::cout << n << "\t" << m << std::endl;
      }
    }
    break;
  case interprime:
    {
      isprime_bit_table_class::huge_uint_type m;

      if (n < 3) n = 3;
      n = isprime_bit_table.firstprime(n);
      m = isprime_bit_table.nextprime(n);
      for (; m < u; n = m, m = isprime_bit_table.nextprime(n))
	std::cout << (n + m)/2 << std::endl;
    }
    break;
  case twin_prime:
  case cousin_primes:
  case sexy_primes:
    {
      isprime_bit_table_class::huge_uint_type d, c, i;
      switch (type) {
      case twin_prime:			d = 2; c = 2; break;
      case cousin_primes:		d = 4; c = 2; break;
      case sexy_primes: default:	d = 6; c = 2; break;
      }
      n = isprime_bit_table.firstprime(n);
      for (; n < u; n = isprime_bit_table.nextprime(n)) {
	for (i=1; i<c; ++i) {
	  isprime_bit_table_class::huge_uint_type m(n + d*i);
	  if (!(m < isprime_bit_table.get_upper())) throw std::length_error("isprime_bit_table: over the length");
	  if (!isprime_bit_table[m]) break;
	}
	if (i==c) {
	  for (i=0; i<c; ++i)
	    std::cout << n + d*i << "\t";
	  std::cout << std::endl;
	}
      }
    }
    break;
  case square_free:
    for (; n < u; ++n)
      if (is_square_free(n))
	std::cout << n << std::endl;
    break;
  case cube_free:
    for (; n < u; ++n)
      if (is_cube_free(n))
	std::cout << n << std::endl;
    break;
  case perfect_square:
    for (; n<u; ++n)
      if (is_perfect_square(n))
	std::cout << n << std::endl;
    break;
  case perfect_power:
    for (; n<u; ++n)
      if (is_perfect_power(n))
	std::cout << n << std::endl;
    break;
  case powerful:
    for (; n<u; ++n)
      if (is_powerful(n))
	std::cout << n << std::endl;
    break;
  case highlycomposite:
    {
      isprime_bit_table_class::huge_uint_type m(0), c;

      for (n = 1; n < 3 && n < u; ++n)
	if ((c=n_divisors(n)) > m) {
	  if (!(n < l)) std::cout << n << std::endl;
	  m = c;
	}
      for (; n < u; n = isprime_bit_table.nextcomposite(n))
	if ((c=n_divisors(n)) > m) {
	  if (!(n < l)) std::cout << n << std::endl;
	  m = c;
	}
    }
    break;
  case highlyabundant:
    {
      isprime_bit_table_class::huge_uint_type m(0), s;

      for (n = 1; n < 3 && n < u; ++n)
	if ((s=sum_divisors(n)) > m) {
	  if (!(n < l)) std::cout << n << std::endl;
	  m = s;
	}
      for (; n < u; n = isprime_bit_table.nextcomposite(n))
	if ((s=sum_divisors(n)) > m) {
	  if (!(n < l)) std::cout << n << std::endl;
	  m = s;
	}
    }
    break;
  case superabundant:
    {
      isprime_bit_table_class::huge_uint_type m(0), s, o(1);

      for (n = 1; n < 3 && n < u; ++n)
	if ((s=sum_divisors(n))*o > m*n) {
	  if (!(n < l)) std::cout << n << std::endl;
	  m = s;
	  o = n;
	}
      for (; n < u; n = isprime_bit_table.nextcomposite(n))
	if ((s=sum_divisors(n))*o > m*n) {
	  if (!(n < l)) std::cout << n << std::endl;
	  m = s;
	  o = n;
	}
    }
    break;
  case abundant:
    for (n = isprime_bit_table.firstcomposite(n); n < u; n = isprime_bit_table.nextcomposite(n))
      if (is_abundant(n))
	std::cout << n << std::endl;
    break;
  case primitive_abundant:
    for (n = isprime_bit_table.firstcomposite(n); n < u; n = isprime_bit_table.nextcomposite(n))
      if (is_primitive_abundant(n))
	std::cout << n << std::endl;
    break;
  case primitive_abundant_have_no_abundant_proper_divisor:
    for (n = isprime_bit_table.firstcomposite(n); n < u; n = isprime_bit_table.nextcomposite(n))
      if (is_primitive_abundant_have_no_abundant_proper_divisor(n))
	std::cout << n << std::endl;
    break;
  case non_prime_unusual:
    for (n = isprime_bit_table.firstcomposite(n); n < u; n = isprime_bit_table.nextcomposite(n))
      if (is_non_prime_unusual(n))
	std::cout << n << std::endl;
    break;
  case pseudoperfect:
  case semiperfect:
    for (n = isprime_bit_table.firstcomposite(n); n < u; n = isprime_bit_table.nextcomposite(n))
      if (is_pseudoperfect(n))
	std::cout << n << std::endl;
    break;
  case primitive_pseudoperfect:
  case primitive_semiperfect:
    for (n = isprime_bit_table.firstcomposite(n); n < u; n = isprime_bit_table.nextcomposite(n))
      if (is_primitive_pseudoperfect(n))
	std::cout << n << std::endl;
    break;
  case perfect:
    for (; n < u; ++n)
      if (is_perfect(n))
	std::cout << n << std::endl;
    break;
  case multiperfect:
    for (; n < u; ++n)
      if (is_multiperfect(n))
	std::cout << n << std::endl;
    break;
  case deficient:
    for (; n < u; ++n)
      if (is_deficient(n))
	std::cout << n << std::endl;
    break;
  case superperfect:
    for (; n < u; ++n)
      if (is_superperfect(n))
	std::cout << n << std::endl;
    break;
  case hemiperfect:
    for (; n < u; ++n) {
      isprime_bit_table_class::huge_uint_type k;
      if (is_hemiperfect(n, k))
	std::cout << n << "\t" << k << std::endl;
    }
    break;
  case refactorable:
    for (; n < u; ++n)
      if (is_refactorable(n))
	std::cout << n << std::endl;
    break;
  case unitary_perfect:
    for (; n < u; ++n)
      if (is_unitary_perfect(n))
	std::cout << n << std::endl;
    break;
  case primary_pseudoperfect:
    for (; n < u; ++n)
      if (is_primary_pseudoperfect(n))
	std::cout << n << std::endl;
    break;
  case weird:
    for (; n < u; ++n)
      if (is_weird(n))
	std::cout << n << std::endl;
    break;
  case practical:
    for (; n < u; ++n)
      if (is_practical(n))
	std::cout << n << std::endl;
    break;
  case ruth_aaron_pair:
    for (; n < u; ++n)
      if (is_ruth_aaron_pair(n))
	std::cout << n << "\t" << n+1 << std::endl;
    break;
  case primorial_ruth_aaron_pair:
    for (; n < u; ++n)
      if (is_primorial_ruth_aaron_pair(n))
	std::cout << n << "\t" << n+1 << std::endl;
    break;
  case unusual:
    for (; n < u; ++n)
      if (is_unusual(n))
	std::cout << n << std::endl;
    break;
  case arithmetic:
    for (; n < u; ++n) {
      isprime_bit_table_class::huge_uint_type s, c;
      if (is_arithmetic(n, s, c))
	std::cout << n << "\t" << s/c << std::endl;
    }
    break;
  case harmonic_divisor:
    for (; n < u; ++n)
      if (is_harmonic_divisor(n))
	std::cout << n << std::endl;
    break;
  case round_number:
    for (; n < u; ++n)
      if (is_round_number(n))
	std::cout << n << std::endl;
    break;
  case lucas_carmichael:
    for (; n < u; ++n)
      if (is_lucas_carmichael(n))
	std::cout << n << std::endl;
    break;
  case semiprime:
    {
      isprime_bit_table_class::huge_uint_type i, j;

      for (; n < u; ++n)
	for (i = 2; i < u; i = isprime_bit_table.nextprime(i))
	  for (j = 2; j < u/i; j = isprime_bit_table.nextprime(j))
	    if (n == i*j) {
	      std::cout << n << "\t" << i << "\t" << j << std::endl;
	      i = u;
	      break;
	    }
    }
    break;
  case sphenic:
    {
      isprime_bit_table_class::huge_uint_type i, j, k;

      for (; n < u; ++n) {
	for (i = 2; i < u; i = isprime_bit_table.nextprime(i))
	  for (j = isprime_bit_table.nextprime(i); j < u/i; j = isprime_bit_table.nextprime(j))
	    for (k = isprime_bit_table.nextprime(j); k < u/i/j; k = isprime_bit_table.nextprime(k))
	      if (n == i*j*k) {
		std::cout << n << "\t" << i << "\t" << j << "\t" << k << std::endl;
		i = u;
		j = u/i;
		break;
	      }
      }
    }
    break;
  case amicable:
    for (; n < u; ++n) {
      isprime_bit_table_class::huge_uint_type s = sum_divisors(n) - n, m;
      for (m = n + 1; m < u; ++m)
	if (s == m && sum_divisors(m) - m == n)
	  std::cout << n << "\t" << m << std::endl;
    }
    break;
  case betrothed:
    for (; n < u; ++n) {
      isprime_bit_table_class::huge_uint_type s = sum_divisors(n) - n - 1, m;
      for (m = n + 1; m < u; ++m)
	if (s == m && sum_divisors(m) - m - 1 == n)
	  std::cout << n << "\t" << m << std::endl;
    }
    break;
  case augumented_amicable:
    for (; n < u; ++n) {
      isprime_bit_table_class::huge_uint_type s = sum_divisors(n) - n + 1, m;
      for (m = n + 1; m < u; ++m)
	if (s == m && sum_divisors(m) - m + 1 == n)
	  std::cout << n << "\t" << m << std::endl;
    }
    break;
  case friendly:
    for (; n < u; ++n) {
      isprime_bit_table_class::huge_uint_type s1 = sum_divisors(n), c1 = boost::math::gcd(s1, n), d1 = n/c1, m;
      s1 /= c1;
      for (m = n + 1; m < u; ++m) {
	isprime_bit_table_class::huge_uint_type s2 = sum_divisors(m), c2 = boost::math::gcd(s2, m), d2 = m/c2;
	s2 /= c2;
	if (s1 == s2 && d1 == d2)
	  std::cout << n << "\t" << m << std::endl;
      }
    }
    break;
  case mersenne:
    {
      for (isprime_bit_table_class::huge_uint_type i=2; ; i = isprime_bit_table.nextprime(i)) {
	n = function_mersenne(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case mersenne_prime:
    {
      for (isprime_bit_table_class::huge_uint_type i=2; ; i = isprime_bit_table.nextprime(i)) {
	n = function_mersenne(i);
	if (!(n < u)) break;
	if (isprime_bit_table[n])
	  if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case wilson_quotient:
    {
      for (isprime_bit_table_class::huge_uint_type i=2; ; i = isprime_bit_table.nextprime(i)) {
	n = function_wilson_quotient(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case wilson_prime:
    {
      for (isprime_bit_table_class::huge_uint_type i=2; ; i = isprime_bit_table.nextprime(i)) {
	n = function_wilson_quotient(i);
	if (!(n < u)) break;
	if (is_wilson_prime(n, i))
	  if (!(n < l)) std::cout << i << "\t" << n << std::endl;
      }
    }
    break;
  case fortunate:
    {
      isprime_bit_table_class::huge_uint_type m, i, p;

      for (i = 1; ; ++i) {
	p = primorial_number(i);
	for (m = 2; ; ++m) {
	  n = p + m;
	  if (!(n < u)) break;
	  if (isprime_bit_table[n]) {
	    if (!(n < l)) std::cout << m << "\t" << n << std::endl;
	    break;
	  }
	}
	if (!(n < u)) break;
      }
    }
    break;
  case hilbert:
    {
      for (isprime_bit_table_class::huge_uint_type i=0; ; ++i) {
	n = function_hilbert(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case pronic:
    {
      for (isprime_bit_table_class::huge_uint_type i=0; ; ++i) {
	n = function_pronic(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case power_of_two:
    {
      for (unsigned long i=0; ; ++i) {
	n = function_power_of_two(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case cullen:
    {
      for (unsigned long i=0; ; ++i) {
	n = function_cullen(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << "\t" << i << std::endl;
      }
    }
    break;
  case cullen_prime:
    {
      for (unsigned long i=0; ; ++i) {
	n = function_cullen(i);
	if (!(n < u)) break;
	if (isprime_bit_table[n])
	  if (!(n < l)) std::cout << n << "\t" << i << std::endl;
      }
    }
    break;
  case woodall:
    {
      for (unsigned long i=1; ; ++i) {
	n = function_woodall(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << "\t" << i << std::endl;
      }
    }
    break;
  case woodall_prime:
    {
      for (unsigned long i=1; ; ++i) {
	n = function_woodall(i);
	if (!(n < u)) break;
	if (isprime_bit_table[n])
	  if (!(n < l)) std::cout << n << "\t" << i << std::endl;
      }
    }
    break;
  case regular:
    {
#if !0
      for (; n < u; ++n) {
	std::vector<isprime_bit_table_class::huge_uint_type> f = prime_factors(n);
	std::vector<isprime_bit_table_class::huge_uint_type>::iterator it;
	for (it=f.begin(); it!=f.end(); ++it)
	  if (!(*it == 2 || *it == 3 || *it == 5))
	    break;
	if (it == f.end())
	  std::cout << n << std::endl;
      }
#else // ! in order
      isprime_bit_table_class::huge_uint_type m[3];

      for (unsigned long i=0; ; ++i) {
	m[0] = pow(5, i/9);
	m[1] = pow(3, (i%9)/3);
	m[2] = pow(2, (i%3));
	n = m[2]*m[1]*m[0];
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
#endif
    }
    break;
  case factorial_prime:
    {
      for (unsigned long i=0; ; ++i) {
	isprime_bit_table_class::huge_uint_type f = factorial(i);
	n = f - 1;
	if (!(n < u)) break;
	if (isprime_bit_table[n])
	  if (!(n < l)) std::cout << n << std::endl;
	n = f + 1;
	if (!(n < u)) break;
	if (isprime_bit_table[n])
	  if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case euclid:
    {
      for (isprime_bit_table_class::huge_uint_type i=1; ; ++i) {
	n = function_euclid(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case euclid_prime:
    {
      for (isprime_bit_table_class::huge_uint_type i=1; ; ++i) {
	n = function_euclid(i);
	if (!(n < u)) break;
	if (isprime_bit_table[n])
	  if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case kummer:
    {
      for (isprime_bit_table_class::huge_uint_type i=1; ; ++i) {
	n = function_kummer(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case kummer_prime:
    {
      for (isprime_bit_table_class::huge_uint_type i=1; ; ++i) {
	n = function_kummer(i);
	if (!(n < u)) break;
	if (isprime_bit_table[n])
	  if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case primorial_prime:
    {
      for (isprime_bit_table_class::huge_uint_type i=1; ; ++i) {
	n = primorial_number(i) - 1;
	if (!(n < u)) break;
	if (isprime_bit_table[n])
	  if (!(n < l)) std::cout << n << "\t" << i << std::endl;
	n += 2;
	if (!(n < u)) break;
	if (isprime_bit_table[n])
	  if (!(n < l)) std::cout << n << "\t" << i << std::endl;
      }
    }
    break;
  case triangular:
    {
      for (isprime_bit_table_class::huge_uint_type i=0; ; ++i) {
	n = function_triangular(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case triangular_pyramidal: case tetrahedral:
    {
      for (isprime_bit_table_class::huge_uint_type i=0; ; ++i) {
	n = function_tetrahedral(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case pentatope:
    {
      for (isprime_bit_table_class::huge_uint_type i=0; ; ++i) {
	n = function_pentatope(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case pentagonal:
    {
      for (isprime_bit_table_class::huge_uint_type i=0; ; ++i) {
	n = function_pentagonal(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case square_triangular:
    {
      for (isprime_bit_table_class::huge_uint_type i=0; ; ++i) {
	n = function_triangular(i);
	if (!(n < u)) break;
	if (is_square_triangular(n))
	  if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case squared_triangular:
    {
      for (isprime_bit_table_class::huge_uint_type i=0; ; ++i) {
	n = function_squared_triangular(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case lazy_caterer:
    {
      for (isprime_bit_table_class::huge_uint_type i=0; ; ++i) {
	n = function_lazy_caterer(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case jacobsthal:
    {
      for (size_t i=0; ; ++i) {
	n = function_jacobsthal(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case jacobsthal_lucas:
    {
      for (size_t i=0; ; ++i) {
	n = function_jacobsthal_lucas(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case kynea:
    {
      for (size_t i=1; ; ++i) {
	n = function_kynea(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case prime_kynea:
    {
      for (size_t i=1; ; ++i) {
	n = function_kynea(i);
	if (!(n < u)) break;
	if (isprime_bit_table[n])
	  if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case thabit:
    {
      for (size_t i=0; ; ++i) {
	n = function_thabit(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case prime_thabit:
    {
      for (size_t i=0; ; ++i) {
	n = function_thabit(i);
	if (!(n < u)) break;
	if (isprime_bit_table[n])
	  if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case fermat: // !
    {
      for (unsigned long i=0; ; ++i) {
	n = function_fermat(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case catalan:
    {
      for (unsigned long i=0; ; ++i) {
	n = function_catalan(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case derangement:
    {
      for (unsigned long i=0; ; ++i) {
	n = subfactorial(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case nontotient:
    {
      isprime_bit_table_class::huge_uint_type i, j;

      for (i=2; ; i+=2) {
	n = i*i;
	if (!(n < u)) break;
	for (j = i; j<n; ++j)
	  if (i == totient_function(j))
	    break;
	if (j == n)
	  if (!(n < l)) std::cout << i << std::endl;
      }
    }
    break;
  case noncototient:
    {
      isprime_bit_table_class::huge_uint_type i, j;

      for (i=4; ; i+=2) {
	n = i*i;
	if (!(n < u)) break;
	for (j = i; j<n; ++j)
	  if (i == cototient_function(j))
	    break;
	if (j == n)
	  if (!(n < l)) std::cout << i << std::endl;
      }
    }
    break;
  case perfect_totient:
    {
      isprime_bit_table_class::huge_uint_type t, s;

      for (; n < u; ++n) {
	t = n;
	s = 0;
	while ((t = totient_function(t)) > 1)
	  s += t;
	if (n != 1) ++s;
	if (s == n)
	  if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case highly_totient:
    {
      isprime_bit_table_class::huge_uint_type i, j, d(0), c;

      for (i=1; ; i++) {
	c = 0;
	n = i*i*2;
	if (!(n < u)) break;
	for (j = i; j<n; ++j)
	  if (i == totient_function(j))
	    ++c;
	if (d < c) {
	  if (!(n < l)) std::cout << i << "\t" << c << std::endl;
	  d = c;
	}
      }
    }
    break;
  case highly_cototient:
    {
      isprime_bit_table_class::huge_uint_type i, j, d(0), c;

      for (i=1; ; ++i) {
	c = 0;
	n = i*i*2;
	if (!(n < u)) break;
	for (j = i; j<n; ++j)
	  if (i == cototient_function(j))
	    ++c;
	if (d < c) {
	  if (!(n < l)) std::cout << i << "\t" << c << std::endl;
	  d = c;
	}
      }
    }
    break;
  case odd_semifactrial:
    {
      isprime_bit_table_class::huge_uint_type i;

      for (i=1; ; i+=2) {
	n = semifactorial(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case even_semifactrial:
    {
      isprime_bit_table_class::huge_uint_type i;

      for (i=0; ; i+=2) {
	n = semifactorial(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
    }
    break;
  case fibonacci:
    {
#if !0 
      for (unsigned long i=0; ; ++i) {
	n = function_fibonacci(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
#else
      isprime_bit_table_class::huge_uint_type F[] = { 0, 1, };
      size_t t;
      for (t=0; t<sizeof(F)/sizeof(F[0]); ++t)
	if (!(F[t] < l)) std::cout << F[t] << std::endl;
      for (; ; ) {
	n = F[0] + F[1];
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
	for (t=0; t+1<sizeof(F)/sizeof(F[0]); ++t) F[t] = F[t+1];
	F[t] = n;
      }
#endif
    }
    break;
  case leonardo:
    {
#if !0
      for (unsigned long i=0; ; ++i) {
	n = function_leonardo(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
#else
      isprime_bit_table_class::huge_uint_type L[] = { 1, 1, };
      size_t t;
      for (t=0; t<sizeof(L)/sizeof(L[0]); ++t)
	if (!(L[t] < l)) std::cout << L[t] << std::endl;
      for (; ; ) {
	n = L[0] + L[1] + 1;
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
	for (t=0; t+1<sizeof(L)/sizeof(L[0]); ++t) L[t] = L[t+1];
	L[t] = n;
      }
#endif
    }
    break;
  case lucas:
    {
#if !0
      for (unsigned long i=0; ; ++i) {
	n = function_lucas(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
#else
      isprime_bit_table_class::huge_uint_type L[] = { 2, 1, };
      size_t t;
      for (t=0; t<sizeof(L)/sizeof(L[0]); ++t)
	if (!(L[t] < l)) std::cout << L[t] << std::endl;
      for (; ; ) {
	n = L[0] + L[1];
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
	for (t=0; t+1<sizeof(L)/sizeof(L[0]); ++t) L[t] = L[t+1];
	L[t] = n;
      }
#endif
    }
    break;
  case pell:
    {
#if !0
      for (unsigned long i=0; ; ++i) {
	n = function_pell(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
#else
      isprime_bit_table_class::huge_uint_type P[] = { 0, 1, };
      size_t t;
      for (t=0; t<sizeof(P)/sizeof(P[0]); ++t)
	if (!(P[t] < l)) std::cout << P[t] << std::endl;
      for (; ; ) {
	n = P[1]*2 + P[0];
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
	for (t=0; t+1<sizeof(P)/sizeof(P[0]); ++t) P[t] = P[t+1];
	P[t] = n;
      }
#endif
    }
    break;
  case perrin:
    {
      isprime_bit_table_class::huge_uint_type P[] = { 3, 0, 2, };
      size_t t;
      for (t=0; t<sizeof(P)/sizeof(P[0]); ++t)
	if (!(P[t] < l)) std::cout << P[t] << std::endl;
      for (; ; ) {
	n = P[1] + P[0];
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
	for (t=0; t+1<sizeof(P)/sizeof(P[0]); ++t) P[t] = P[t+1];
	P[t] = n;
      }
    }
    break;
  case padovan:
    {
      isprime_bit_table_class::huge_uint_type P[] = { 1, 1, 1, };
      size_t t;
      for (t=0; t<sizeof(P)/sizeof(P[0]); ++t)
	if (!(P[t] < l)) std::cout << P[t] << std::endl;
      for (; ; ) {
	n = P[0] + P[1];
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
	for (t=0; t+1<sizeof(P)/sizeof(P[0]); ++t) P[t] = P[t+1];
	P[t] = n;
      }
    }
    break;
  case motzkin:
    {
#if 0
      for (unsigned long i=0; ; ++i) {
	n = function_motzkin(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
#else
      isprime_bit_table_class::huge_uint_type P[] = { 1, 1, };
      size_t t;
      for (t=0; t<sizeof(P)/sizeof(P[0]); ++t)
	if (!(P[t] < l)) std::cout << P[t] << std::endl;
      for (isprime_bit_table_class::huge_uint_type i=2; ; ++i) {
	n = (P[0]*3*(i-1) + P[1]*(2*i+1))/(2+i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
	for (t=0; t+1<sizeof(P)/sizeof(P[0]); ++t) P[t] = P[t+1];
	P[t] = n;
      }
#endif
    }
    break;
  case schreder_hipparchus:
    {
      isprime_bit_table_class::huge_uint_type S[] = { 1, 1, };
      size_t t;
      for (t=0; t<sizeof(S)/sizeof(S[0]); ++t)
	if (!(S[t] < l)) std::cout << S[t] << std::endl;
      for (isprime_bit_table_class::huge_uint_type i=3; ; ++i) {
	n = ((6*i - 9)*S[1] - (i - 3)*S[0])/i;
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
	for (t=0; t+1<sizeof(S)/sizeof(S[0]); ++t) S[t] = S[t+1];
	S[t] = n;
      }
    }
    break;
  case sylvester:
    {
      isprime_bit_table_class::huge_uint_type S[] = { 2, };
      size_t t;
      for (t=0; t<sizeof(S)/sizeof(S[0]); ++t)
	if (!(S[t] < l)) std::cout << S[t] << std::endl;
      for (isprime_bit_table_class::huge_uint_type i=1; ; ++i) {
	n = S[0]*(S[0] - 1) + 1;
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
	for (t=0; t+1<sizeof(S)/sizeof(S[0]); ++t) S[t] = S[t+1];
	S[t] = n;
      }
    }
    break;
  case telephone:
    {
#if 0
      for (unsigned long i=0; ; ++i) {
	n = function_telephone(i);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
      }
#else
      isprime_bit_table_class::huge_uint_type T[] = { 1, 1, };
      size_t t;
      for (t=0; t<sizeof(T)/sizeof(T[0]); ++t)
	if (!(T[t] < l)) std::cout << T[t] << std::endl;
      for (isprime_bit_table_class::huge_uint_type i=2; ; ++i) {
	n = T[1] + (i - 1)*T[0];
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
	for (t=0; t+1<sizeof(T)/sizeof(T[0]); ++t) T[t] = T[t+1];
	T[t] = n;
      }
#endif
    }
    break;
  case alternating_factorial:
    {
      isprime_bit_table_class::huge_uint_type F[] = { 1, };
      size_t t;
      for (t=0; t<sizeof(F)/sizeof(F[0]); ++t)
	if (!(F[t] < l)) std::cout << F[t] << std::endl;
      for (unsigned long i=2; ; ++i) {
	n = factorial(i) - F[0];
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
	for (t=0; t+1<sizeof(F)/sizeof(F[0]); ++t) F[t] = F[t+1];
	F[t] = n;
      }
    }
    break;
  case hyperfactorial:
    {
      isprime_bit_table_class::huge_uint_type F[] = { 1, };
      size_t t;
      for (t=0; t<sizeof(F)/sizeof(F[0]); ++t)
	if (!(F[t] < l)) std::cout << F[t] << std::endl;
      for (unsigned long i=2; ; ++i) {
	n = pow(i, i)*F[0];
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
	for (t=0; t+1<sizeof(F)/sizeof(F[0]); ++t) F[t] = F[t+1];
	F[t] = n;
      }
    }
    break;
  case exponential_factorial:
    {
      isprime_bit_table_class::huge_uint_type F[] = { 1, };
      size_t t;
      for (t=0; t<sizeof(F)/sizeof(F[0]); ++t)
	if (!(F[t] < l)) std::cout << F[t] << std::endl;
      for (unsigned long i=1; ; ++i) {
	n = pow(i, F[0]);
	if (!(n < u)) break;
	if (!(n < l)) std::cout << n << std::endl;
	for (t=0; t+1<sizeof(F)/sizeof(F[0]); ++t) F[t] = F[t+1];
	F[t] = n;
      }
    }
    break;
  default:
    break;
  }

  return 0;
}
