/*
  prime_util.hh

  Copyright (C) 2015 Taiji Yamada <taiji@aihara.co.jp>
*/
#ifndef _PRIME_UTIL_HH_
#define _PRIME_UTIL_HH_
#include <iostream>
#include <fstream>
#include <vector>
#include <cmath>
#include <boost/interprocess/file_mapping.hpp>
#include <boost/interprocess/mapped_region.hpp>
#include <boost/math/common_factor.hpp>
#include <boost/math/special_functions/binomial.hpp>
#include <stdexcept>

template <typename T>
inline
T function_fibonacci(const T i)
{
  return floor(2*(pow((1+sqrt(5))/2, i) + 1) / (2*sqrt(5)));
}
template <typename T>
inline
T function_leonardo(const T i)
{
  return function_fibonacci(i+1)*2 - 1;
}
template <typename T>
inline
T function_lucas(const T i)
{
  return pow((1+sqrt(5))/2, i) + pow((1-sqrt(5))/2, i);
}
template <typename T>
inline
T function_pell(const T i)
{
  return (pow((1+sqrt(2)), i) + pow((1-sqrt(2)), i))/(2*sqrt(2));
}

template <typename T>
inline
T pow(const T n, const T m)
{
  T p(1);
  for (size_t i=0; i<m; ++i)
    p *= n;
  return p;
}

template <typename T>
inline
T function_hilbert(const T i)
{
  return 4*i + 1;
}
template <typename T>
inline
T function_pronic(const T i)
{
  return i*(i+1);
}
template <typename T>
inline
T function_power_of_two(const T i)
{
  T n;

  n = pow(2, i);
  return n;
}
template <typename T>
inline
T function_cullen(const T i)
{
  return function_power_of_two(i)*i + 1;
}
template <typename T>
inline
T function_woodall(const T i)
{
  return function_power_of_two(i)*i - 1;
}
template <typename T>
inline
T function_triangular(const T i)
{
  return (i * (i+1))/2;
}
template <typename T>
inline
T function_tetrahedral(const T i)
{
  return (i * (i+1) * (i+2))/6;
}
template <typename T>
inline
T function_pentatope(const T i)
{
  return (i * (i+1) * (i+2) * (i+3))/24;
}
template <typename T>
inline
T function_pentagonal(const T i)
{
  return (i * (i*3-1))/2;
}
template <typename T>
inline
bool is_square_triangular(const T n)
{
  T r1, r2;

  r1 = sqrt(n);
  r2 = n - r1*r1;
  return r2 == 0;
}
template <typename T>
inline
T function_squared_triangular(const T i)
{
  return ((i*i + i)/2)*((i*i + i)/2);
}
template <typename T>
inline
T function_lazy_caterer(const T i)
{
  return (i*i + i + 2)/2;
}
template <typename T>
inline
T function_jacobsthal(const T i)
{
  return (function_power_of_two(i) - ((i%2)?-1:1))/3;
}
template <typename T>
inline
T function_jacobsthal_lucas(const T i)
{
  return function_power_of_two(i) + ((i%2)?-1:1);
}
template <typename T>
inline
T function_kynea(const T i)
{
  T n(function_power_of_two(i) + 1);

  return n*n - 2;
}
template <typename T>
inline
T function_thabit(const T i)
{
  return 3*function_power_of_two(i) - 1;
}
template <typename T>
inline
T function_fermat(const T i)
{
  return function_power_of_two(function_power_of_two(i)) + 1; // ! too huge
}
template <typename T>
inline
T function_catalan(const T i)
{
  return boost::math::binomial_coefficient<double>(2*i, i) / (i+1);
}
template <typename T>
inline
T function_motzkin(const T i) // ! wrong
{
  T n(0);

  for (unsigned long j=0; j<i/2; ++j)
    n += boost::math::binomial_coefficient<double>(i, j*2)*function_catalan(j);
  return n;
}

template <typename T>
bool is_square_free(const T n)
{
  T i, sqrt_n;

  sqrt_n = sqrt(n);
  for (i=2; i<=sqrt_n; ++i)
    if (n % (i*i) == 0)
      break;
  return i == sqrt_n+1;
}
template <typename T>
bool is_cube_free(const T n)
{
  T i, cbrt_n;

  cbrt_n = cbrt(n);
  for (i=2; i<=cbrt_n; ++i)
    if (n % (i*i*i) == 0)
      break;
  return i == cbrt_n+1;
}

template <typename T>
std::vector<T> divisors(const T n)
{
  T d(1), sqrt_n;
  std::vector<T> q;

  sqrt_n = sqrt(n);
  for (; d<=sqrt_n; ++d)
    if (n%d == 0) {
      q.push_back(d);
      if (d != n/d) q.push_back(n/d);
    }
  return q;
}
template <typename T>
T n_divisors(const T n)
{
  T d(1), sqrt_n, c(0);

  sqrt_n = sqrt(n);
  for (; d<=sqrt_n; ++d)
    if (n%d == 0) {
      ++c;
      if (d != n/d) ++c;
    }
  return c;
}
template <typename T>
T sum_divisors(const T n)
{
  T d(1), sqrt_n, s(0);

  sqrt_n = sqrt(n);
  for (; d<=sqrt_n; ++d)
    if (n%d == 0) {
      s += d;
      if (d != n/d) s += n/d;
    }
  return s;
}

template <typename T>
inline
bool is_abundant(const T n)
{
  return sum_divisors(n) > n*2;
}
template <typename T>
inline
bool is_perfect(const T n)
{
  return sum_divisors(n) - n == n;
}
template <typename T>
inline
bool is_multiperfect(const T n)
{
  return sum_divisors(n) % n == 0;
}
template <typename T>
inline
bool is_deficient(const T n)
{
  return sum_divisors(n) < n*2;
}
template <typename T>
inline
bool is_superperfect(const T n)
{
  return sum_divisors(sum_divisors(n)) == n*2;
}
template <typename T>
inline
bool is_hemiperfect(const T n, T &k)
{
  T s;
  return (s=sum_divisors(n))*2 % n == 0 && (k = s*2/n) % 2 == 1;
}
template <typename T>
inline
bool is_refactorable(const T n)
{
  return n % n_divisors(n) == 0;
}
template <typename T>
inline
bool is_arithmetic(const T n, T &s, T &c)
{
  return (s=sum_divisors(n)) % (c=n_divisors(n)) == 0;
}

template <typename T>
bool is_harmonic_divisor(const T n)
{
  std::vector<T> q = divisors(n);
  T d(0), c(q[0]);
  size_t i;

  for (i=1; i<q.size(); ++i)
    c = boost::math::lcm(c, q[i]);
  for (i=0; i<q.size(); ++i)
    d += c/q[i];
  return (T(q.size())*c) % d == 0;
}

template <typename T>
std::vector<T> proper_divisors(const T n)
{
  T d(1), sqrt_n;
  std::vector<T> q;

  sqrt_n = sqrt(n);
  for (; d<=sqrt_n; ++d)
    if (n%d == 0) {
      if (n != d) q.push_back(d);
      if (n != n/d && d != n/d) q.push_back(n/d);
    }
  return q;
}
template <typename T>
T n_proper_divisors(const T n)
{
  T d(1), sqrt_n, c(0);

  sqrt_n = sqrt(n);
  for (; d<=sqrt_n; ++d)
    if (n%d == 0) {
      if (n != d) ++c;
      if (n != n/d && d != n/d) ++c;
    }
  return c;
}
template <typename T>
T sum_proper_divisors(const T n)
{
  T d(1), sqrt_n, s(0);

  sqrt_n = sqrt(n);
  for (; d<=sqrt_n; ++d)
    if (n%d == 0) {
      if (n != d) s += d;
      if (n != n/d && d != n/d) s += n/d;
    }
  return s;
}

template <typename T>
bool is_primitive_abundant(const T n)
{
  if (!is_abundant(n)) return false;
  std::vector<T> d = proper_divisors(n);
  typename std::vector<T>::iterator it;

  for (it=d.begin(); it!=d.end(); ++it)
    if (!(sum_divisors(*it) < *it*2))
      break;
  return it == d.end();
}
template <typename T>
bool is_primitive_abundant_have_no_abundant_proper_divisor(const T n)
{
  if (!is_abundant(n)) return false;
  std::vector<T> d = proper_divisors(n);
  typename std::vector<T>::iterator it;

  for (it=d.begin(); it!=d.end(); ++it)
    if ((sum_divisors(*it) > *it*2))
      break;
  return it == d.end();
}
template <typename T>
bool is_pseudoperfect(const T n)
{
  std::vector<T> d = proper_divisors(n);
  T i, c;
  size_t j;

  c = 1U<<d.size();
  for (i=1; i<c; ++i) {
    T s(0);
    for (j=0; j<d.size(); ++j)
      if (i & 1U<<j)
	s += d[j];
    if (s == n)
      return true;
  }
  return false;
}
template <typename T>
bool is_pseudoperfect(const T n, std::vector<T> &d)
{
  T i, c;
  size_t j;

  d = proper_divisors(n);
  c = 1U<<d.size();
  for (i=1; i<c; ++i) {
    T s(0);
    for (j=0; j<d.size(); ++j)
      if (i & 1U<<j)
	s += d[j];
    if (s == n)
      return true;
  }
  return false;
}
template <typename T>
bool is_weird(const T n)
{
  if (!(sum_divisors(n) > n*2)) return false;
  std::vector<T> d = proper_divisors(n);
  T i, c;

  c = 1U<<d.size();
  for (i=1; i<c; ++i) {
    T s(0);
    for (size_t j=0; j<d.size(); ++j)
      if (i & 1U<<j)
	s += d[j];
    if (s == n) break;
  }
  return i == c;
}
template <typename T>
bool is_practical(const T n)
{
  std::vector<T> d = proper_divisors(n);
  T m, i, c;

  c = 1U<<d.size();
  for (m = 1; m < n; ++m) {
    for (i=1; i<c; ++i) {
      T s(0);
      for (size_t j=0; j<d.size(); ++j)
	if (i & 1U<<j)
	  s += d[j];
      if (s == m) break;
    }
    if (i == c) break;
  }
  return !(m < n);
}

template <typename T>
std::vector<T> unitary_divisors(const T n)
{
  T d(1), sqrt_n;
  std::vector<T> q;

  sqrt_n = sqrt(n);
  for (; d<=sqrt_n; ++d)
    if (n%d == 0) {
      if (boost::math::gcd(n/d, d) == 1) {
	q.push_back(d);
	if (d != 1) q.push_back(n/d);
      }
    }
  return q;
}
template <typename T>
T n_unitary_divisors(const T n)
{
  T d(1), sqrt_n, c(0);

  sqrt_n = sqrt(n);
  for (; d<=sqrt_n; ++d)
    if (n%d == 0) {
      if (boost::math::gcd(n/d, d) == 1) {
	++c;
	if (d != 1) ++c;
      }
    }
  return c;
}
template <typename T>
T sum_unitary_divisors(const T n)
{
  T d(1), sqrt_n, s(0);

  sqrt_n = sqrt(n);
  for (; d<=sqrt_n; ++d)
    if (n%d == 0) {
      if (boost::math::gcd(n/d, d) == 1) {
	s += d;
	if (d != 1) s += n/d;
      }
    }
  return s;
}

template <typename T>
inline
bool is_unitary_perfect(const T n)
{
  return n != 1 && sum_unitary_divisors(n) == n;
}

template <typename T>
T factorial(T n)
{
  T f(1);

  if (n == 0) return 1;
  for (; n != 1; --n)
    f *= n;
  return f;
}
template <typename T>
T subfactorial(const T n)
{
  T f(1), i;

  for (i=1; i<=n; ++i)
    f = i*f + ((i%2)?-1:1);
  return f;
}
template <typename T>
T semifactorial(const T n)
{
  T f(1), i((n % 2 == 1) ? 1 : 2);

  for (; i<=n; i+=2)
    f *= i;
  return f;
}

template <typename T>
T function_telephone(const T i)
{
  T n(0);

  for (unsigned long j=0; j<n/2; ++j)
    n += boost::math::binomial_coefficient<double>(i, j*2)*semifactorial(j*2 - 1);
  return n;
}

#if !defined(USE_128_BIT_TYPE)
#if defined(DONT_USE_EVEN_OMISSION_IN_BIT_TABLE)
#define isprime_bit_table_class_file_path "/var/tmp/isprime_bit_table+even.bin"
#else
#define isprime_bit_table_class_file_path "/var/tmp/isprime_bit_table.bin"
#endif
#else
#if defined(DONT_USE_EVEN_OMISSION_IN_BIT_TABLE)
#define isprime_bit_table_class_file_path "/var/tmp/isprime_bit_table+even.128bit.bin"
#else
#define isprime_bit_table_class_file_path "/var/tmp/isprime_bit_table.128bit.bin"
#endif
#endif

class isprime_bit_table_class {
public:
#if !defined(USE_128_BIT_TYPE)
  typedef unsigned long long int huge_uint_type;
  typedef long long int huge_int_type;
  typedef unsigned long long int element_type;
#else
  typedef __uint128_t huge_uint_type;
  typedef __int128_t huge_int_type;
  typedef __uint128_t element_type;
#endif
  huge_uint_type number(size_t z) const
  {
#if defined(DONT_USE_EVEN_OMISSION_IN_BIT_TABLE)
    return z*8;
#else
    return z*16;
#endif
  }
  static const size_t bit_length = sizeof(element_type)*8;
  size_t table_length(huge_uint_type n) const
  {
#if defined(DONT_USE_EVEN_OMISSION_IN_BIT_TABLE)
    return n/bit_length + ((n%bit_length) ? 1 : 0);
#else
    return (n>>1)/bit_length + (((n>>1)%bit_length) ? 1 : 0);
#endif
  }
  huge_uint_type real_upper(huge_uint_type n) const
  {
#if defined(DONT_USE_EVEN_OMISSION_IN_BIT_TABLE)
    return bit_length*table_length(n);
#else
    return bit_length*table_length(n)*2;
#endif
  }
  size_t size(huge_uint_type n) const { return sizeof(element_type)*table_length(real_upper(n)); }
  huge_uint_type get_upper() const { return n; }
  void set_upper(huge_uint_type upper)
  {
    n = real_upper(upper);
    boost::interprocess::file_mapping mapped_file;
    try {
      mapped_file = boost::interprocess::file_mapping(isprime_bit_table_class_file_path, boost::interprocess::read_only);
    }
    catch (boost::interprocess::interprocess_exception &) {
      std::ofstream ofs(isprime_bit_table_class_file_path, std::ofstream::binary);
      table = new element_type[table_length(n)];
      eratosthenes_sieve();
      ofs.write((char *)table, size(n));
      ofs.close();
      delete[] table;
      mapped_file = boost::interprocess::file_mapping(isprime_bit_table_class_file_path, boost::interprocess::read_only);
    }
    mapped_file_region = boost::interprocess::mapped_region(mapped_file, boost::interprocess::read_only);
    table = static_cast<element_type *>(mapped_file_region.get_address());
    huge_uint_type u = number(mapped_file_region.get_size());
    if (n > u) {
      std::ofstream ofs(isprime_bit_table_class_file_path, std::ofstream::app|std::ofstream::binary);
      ofs.seekp(0, std::ios_base::end);
      table = new element_type[table_length(n)];
      continued_eratosthenes_sieve(u);
      ofs.write((char *)&table[table_length(u)], size(n-u));
      ofs.close();
      delete[] table;
      mapped_file = boost::interprocess::file_mapping(isprime_bit_table_class_file_path, boost::interprocess::read_only);
      mapped_file_region = boost::interprocess::mapped_region(mapped_file, boost::interprocess::read_only);
      table = static_cast<element_type *>(mapped_file_region.get_address());
    }
    else
      n = u;
  }
  isprime_bit_table_class(huge_uint_type upper = 200000000) : n(upper) { set_upper(n); }
#if defined(DONT_USE_EVEN_OMISSION_IN_BIT_TABLE)
  bool operator[](const huge_uint_type i) const
  {
    return (table[i/bit_length]&(1ULL<<(i%bit_length))) != 0ULL;
  }
  void set(const huge_uint_type i)
  {
    table[i/bit_length] |= (1ULL<<(i%bit_length));
  }
  void unset(const huge_uint_type i)
  {
    table[i/bit_length] &= ~(1ULL<<(i%bit_length));
  }
#else
  bool operator[](const huge_uint_type i) const
  {
    return (!(i&1)) ? (i == 2) :
      (table[(i>>1)/bit_length]&(1ULL<<((i>>1)%bit_length))) != 0ULL;
  }
  void set(const huge_uint_type i)
  {
    if (!(i&1)) return;
    table[(i>>1)/bit_length] |= (1ULL<<((i>>1)%bit_length));
  }
  void unset(const huge_uint_type i)
  {
    if (!(i&1)) return;
    table[(i>>1)/bit_length] &= ~(1ULL<<((i>>1)%bit_length));
  }
#endif
  void eratosthenes_sieve()
  {
    huge_uint_type i, j;
    const double sqrt_n = sqrt(n);

    memset(table, ~0, size(n));
    unset(0);
    unset(1);
    for (i = 2; i < sqrt_n; i++)
      for (j = 2; i * j < n; j++)
	unset(i*j);
  }
  void continued_eratosthenes_sieve(const huge_uint_type u)
  {
    huge_uint_type i, j;
    const double sqrt_n = sqrt(n);

    memset(&table[table_length(u)], ~0, size(n-u));
    unset(0);
    unset(1);
    for (i = 2; i < sqrt_n; i++)
      for (j = u/i; i * j < n; j++)
	unset(i*j);
  }
  huge_uint_type firstprime(huge_uint_type i)
  {
    if ((*this)[i]) return i;
    while ((++i<n) && !(*this)[i]) {}
    return i;
  }
  huge_uint_type nextprime(huge_uint_type i)
  {
    while ((++i<n) && !(*this)[i]) {}
    return i;
  }
  huge_uint_type previousprime(huge_uint_type i)
  {
    while ((--i>0) && !(*this)[i]) {}
    return i; // 0 means an unnatural number and decrement terminator
  }
  huge_uint_type firstcomposite(huge_uint_type i)
  {
    if (i != 1 && !(*this)[i]) return i;
    while ((++i<n) && (*this)[i]) {}
    return i;
  }
  huge_uint_type nextcomposite(huge_uint_type i)
  {
    while ((++i<n) && (*this)[i]) {}
    return i;
  }
  huge_uint_type previouscomposite(huge_uint_type i)
  {
    while ((--i>0) && !(i != 1 && !(*this)[i])) {}
    return i; // 0 means an unnatural number and decrement terminator
  }
private:
  huge_uint_type n;
  boost::interprocess::mapped_region mapped_file_region;
  element_type *table;
};

template <typename T>
std::vector<T> factorize(T n)
{
  T f(2);
  std::vector<T> p;
  isprime_bit_table_class isprime_bit_table;

  for (; f <= n; f = isprime_bit_table.nextprime(f)) {
    while (n % f == 0) {
      p.push_back(f);
      n /= f;
    }
    if (!(f < isprime_bit_table.get_upper())) throw std::length_error("isprime_bit_table: over the length");
  }
  return p;
}
template <typename T>
T sum_factorize(T n)
{
  T f(2), s(0);
  isprime_bit_table_class isprime_bit_table;

  for (; f <= n; f = isprime_bit_table.nextprime(f)) {
    while (n % f == 0) {
      s += f;
      n /= f;
    }
    if (!(f < isprime_bit_table.get_upper())) throw std::length_error("isprime_bit_table: over the length");
  }
  return s;
}

template <typename T>
bool is_primitive_pseudoperfect(T n)
{
  std::vector<T> d;
  size_t k;
  isprime_bit_table_class isprime_bit_table;

  if (!is_pseudoperfect(n, d)) return false;
  for (k=0; k<d.size(); ++k) {
    if (isprime_bit_table[d[k]]) continue;
    std::vector<T> pd = proper_divisors(d[k]);
    T i, c;
    size_t j;
    c = 1U<<pd.size();
    for (i=1; i<c; ++i) {
      T s(0);
      for (j=0; j<pd.size(); ++j)
	if (i & 1U<<j)
	  s += pd[j];
      if (s == d[k]) break;
    }
    if (i != c) break;
  }
  return k == d.size();
}
template <typename T>
bool is_non_prime_unusual(T n)
{
  std::vector<T> f = factorize(n);
  T sqrt_n;

  sqrt_n = sqrt(n);
  return f.size() > 1 && f[f.size()-1] > sqrt_n;
}
template <typename T>
bool is_unusual(const T n)
{
  std::vector<T> f = factorize(n);
  T sqrt_n;

  sqrt_n = sqrt(n);
  return f.size() > 0 && f[f.size()-1] > sqrt_n;
}

template <typename T>
std::vector<T> prime_factors(T n)
{
  T f(2);
  std::vector<T> p;
  isprime_bit_table_class isprime_bit_table;

  for (; f <= n; f = isprime_bit_table.nextprime(f)) {
    if (n % f == 0) {
      p.push_back(f);
      n /= f;
    }
    while (n % f == 0)
      n /= f;
    if (!(f < isprime_bit_table.get_upper())) throw std::length_error("isprime_bit_table: over the length");
  }
  return p;
}

template <typename T>
bool is_round_number(const T n)
{
  T sqrt_n;
  std::vector<T> f = prime_factors(n);
  typename std::vector<T>::iterator it;

  sqrt_n = sqrt(n);
  for (it=f.begin(); it!=f.end(); ++it)
    if (*it > sqrt_n)
      break;
  return it==f.end();
}
template <typename T>
bool is_lucas_carmichael(const T n)
{
  std::vector<T> f = prime_factors(n);
  typename std::vector<T>::iterator it;

  if (f.size() < 3) return false;
  for (it=f.begin(); it!=f.end(); ++it)
    if ((n+1) % (*it+1) != 0)
      break;
  return it==f.end();
}

template <typename T>
T primorial_number(T n)
{
  T p(2), f(1), i(1);
  isprime_bit_table_class isprime_bit_table;

  for (; i<=n; p = isprime_bit_table.nextprime(p), ++i) {
    f *= p;
    if (!(p < isprime_bit_table.get_upper())) throw std::length_error("isprime_bit_table: over the length");
  }
  return f;
}
template <typename T>
T primorial(T p)
{
  T n(2), f(1);
  isprime_bit_table_class isprime_bit_table;

  for (; n<=p; n = isprime_bit_table.nextprime(n)) {
    f *= n;
    if (!(n < isprime_bit_table.get_upper())) throw std::length_error("isprime_bit_table: over the length");
  }
  return f;
}

template <typename T>
inline
T function_euclid(const T i)
{
  return primorial_number(i) + 1;
}
template <typename T>
inline
T function_kummer(const T i)
{
  return primorial_number(i) - 1;
}

template <typename T>
inline
bool is_ruth_aaron_pair(const T n)
{
  return sum_factorize(n) == sum_factorize(n+1);
}
template <typename T>
bool is_primorial_ruth_aaron_pair(const T n)
{
  std::vector<T> f = factorize(n);

  return sum_factorize(n) == sum_factorize(n+1) &&
    n*(n+1) == primorial(f[f.size()-1]);
}

template <typename T>
bool is_powerful(const T n)
{
  T p(2);
  isprime_bit_table_class isprime_bit_table;

  for (; p<n; p = isprime_bit_table.nextprime(p)) {
    if (n % p == 0 && n % (p*p) == 0)
      return true;
    if (!(p < isprime_bit_table.get_upper())) throw std::length_error("isprime_bit_table: over the length");
  }
  return false;
}
template <typename T>
bool is_perfect_square(T n)
{
  T f(2), c, c0(0);
  isprime_bit_table_class isprime_bit_table;

  for (; f <= n; f = isprime_bit_table.nextprime(f)) {
    c = 0;
    while (n % f == 0) {
      ++c;
      n /= f;
    }
    if (c % 2 == 1)
      return false;
    if (c0 != 0) {
      if (boost::math::gcd(c0, c) % 2 == 1)
	return false;
      c0 = boost::math::gcd(c0, c);
    }
    else
      c0 = c;
    if (!(f < isprime_bit_table.get_upper())) throw std::length_error("isprime_bit_table: over the length");
  }
  return true;
}
template <typename T>
bool is_perfect_power(T n)
{
  T f(2), c, c0(0);
  isprime_bit_table_class isprime_bit_table;

  for (; f <= n; f = isprime_bit_table.nextprime(f)) {
    c = 0;
    while (n % f == 0) {
      ++c;
      n /= f;
    }
    if (c == 1)
      return false;
    if (c0 != 0) {
      if (boost::math::gcd(c0, c) == 1)
	return false;
      c0 = boost::math::gcd(c0, c);
    }
    else
      c0 = c;
    if (!(f < isprime_bit_table.get_upper())) throw std::length_error("isprime_bit_table: over the length");
  }
  return true;
}
template <typename T>
bool is_primary_pseudoperfect(const T n)
{
  std::vector<T> d = divisors(n);
  T s(0);
  typename std::vector<T>::iterator it;
  isprime_bit_table_class isprime_bit_table;

  for (it=d.begin(); it!=d.end(); ++it)
    if (isprime_bit_table[*it])
      s += n/(*it);
  return n != 1 && s + 1 == n;
}

template <typename T>
T totient_function(T n)
{
  T f(2), r(n);
  isprime_bit_table_class isprime_bit_table;

  for (; f <= n; f = isprime_bit_table.nextprime(f)) {
    if (n % f == 0) {
      r -= r / f;
      n /= f;
    }
    while (n % f == 0)
      n /= f;
    if (!(f < isprime_bit_table.get_upper())) throw std::length_error("isprime_bit_table: over the length");
  }
  return r;
}
template <typename T>
T cototient_function(T n)
{
  T f(2), r(n);
  const T c(n);
  isprime_bit_table_class isprime_bit_table;

  for (; f <= n; f = isprime_bit_table.nextprime(f)) {
    if (n % f == 0) {
      r -= r / f;
      n /= f;
    }
    while (n % f == 0)
      n /= f;
    if (!(f < isprime_bit_table.get_upper())) throw std::length_error("isprime_bit_table: over the length");
  }
  return c - r;
}
template <typename T>
T mebius_function(T n)
{
  T f(2), r(1);
  isprime_bit_table_class isprime_bit_table;

  for (; f <= n; f = isprime_bit_table.nextprime(f)) {
    if (n % f == 0) {
      r *= -1;
      n /= f;
    }
    if (n % f == 0)
      return T(0);
    if (!(f < isprime_bit_table.get_upper())) throw std::length_error("isprime_bit_table: over the length");
  }
  return r;
}

template <typename T>
inline
T function_mersenne(const T p)
{
  T m;

  m = 1U<<p;
  return m - 1;
}
template <typename T>
inline
T function_wilson_quotient(const T p)
{
  T f;

  f = factorial(p - 1);
  return (f + 1)/p;
}
template <typename T>
inline
bool is_wilson_prime(const T n, const T p)
{
  return n % p == 0;
}

#endif
